<?php

namespace Ems\portalGestor\Controller;

use Ems\portalGestor\Service\AdmissaoService;
use Ems\portalGestor\Util\ExpectedException;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\{ Error, Message };

class AdmissaoController {
    /** @var AdmissaoService */
    protected $admissaoService;

    public function __construct(
        AdmissaoService $admissaoService
    ) {
        $this->admissaoService = $admissaoService;
    }

    public function getVinculosNaoAprovados(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg     = (int) $row['NRORG'];
            $cdOperador = (string) $row['CDOPERADOR'];

            $nrEstrutura = isset($row['NRESTRUTURAM']) ?
                (int) $row['NRESTRUTURAM'] :
                null;

            $admissoes = $this->admissaoService->getVinculosNaoAprovados(
                $nrOrg, $cdOperador, $nrEstrutura
            );

            $response->addDataSet(new DataSet('data', $admissoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getCategoriasComAdmissaoVinculo(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg     = (int) $row['NRORG'];
            $nrVinculo = (int) $row['NRVINCULOM'];

            $categorias = $this->admissaoService->getCategoriasComAdmissaoVinculo(
                $nrOrg, $nrVinculo
            );

            $response->addDataSet(new DataSet('data', $categorias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getCategorias(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg     = (int) $row['NRORG'];

            $categorias = $this->admissaoService->getCategorias(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $categorias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipos(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $tipos = $this->admissaoService->getTipos(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function downloadAdmissao(
        Request\Filter $request,
        Response       $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg   = (int) $row['NRORG'];
            $nrAnexo = (int) $row['NRANEXO'];

            $file = $this->admissaoService->downloadAdmissao(
                $nrOrg, $nrAnexo
            );

            $response->addDataSet(new DataSet('data', $file));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function setAdmissaoSimplificada(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = (array) $request->getRow()->getArrayCopy();

            $success = $this->admissaoService->setAdmissaoSimplificada(
                $row
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getCursoAcademico(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $cursos = $this->admissaoService->getCursoAcademico(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $cursos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getInstituicaoEstrutura(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $instituicoes = $this->admissaoService->getInstituicaoEstrutura(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $instituicoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipoFormacaoParc(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $tiposFormacao = $this->admissaoService->getTipoFormacaoParc(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tiposFormacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getAreaFormacao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $areas = $this->admissaoService->getAreaFormacao(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $areas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getBeneficio(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $beneficios = $this->admissaoService->getBeneficio(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipoVinculo(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $tipos = $this->admissaoService->getTipoVinculo(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipoAdmissao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $tipos = $this->admissaoService->getTipoAdmissao(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getVinculoEmpregaticio(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $vinculos = $this->admissaoService->getVinculoEmpregaticio(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $vinculos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipoOcupacao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $tipos = $this->admissaoService->getTipoOcupacao(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getOcupacao(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $ocupacoes = $this->admissaoService->getOcupacao(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $ocupacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipoSalario(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $ocupacoes = $this->admissaoService->getTipoSalario(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $ocupacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function setAdmissaoCompleta(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = (array) $request->getRow()->getArrayCopy();

            $success = $this->admissaoService->setAdmissaoCompleta(
                $row
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function pesquisarCep(
        Request\Filter $request,
        Response    $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int)    $row['NRORG'];
            $cep   = (string) $row['CEP'  ];

            $status = $this->admissaoService->
                pesquisarCep($nrOrg, $cep);

            if (!$status['success']) {
                $message = $status['message'];
                $response->addMessage(new Message($message));
            } else {
                $dataset = $status['cepInfo'];
                $response->addDataSet(new DataSet('data', $dataset));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
